require 'active_support/core_ext'
require 'action_view/helpers'
require 'bonanza/link_strategies'

module Bonanza
  module LinkHelpers
    include ActionView::Helpers

    class InvalidAppError < StandardError; end

    def nine_app_url(app, entity, reference = nil)
      strategy = strategy_from_app_name(app, entity)
      strategy.url_to entity, reference
    end

    def link_to_otrs_ticket(name, ticket_number, options = {})
      link_to_application :otrs, :ticket, name, ticket_number, options
    end

    def link_to_otrs_customer(name, account_number, options = {})
      link_to_application :otrs, :customer, name, account_number, options
    end

    def link_to_billing(name, account_number, options = {})
      link_to_application :billing, :debtor, name, account_number, options
    end

    def link_to_stats(name, account_number, options = {})
      link_to_application :stats, :customer, name, account_number, options
    end

    def link_to_redmine(name, redmine_issue, options = {})
      link_to_application :redmine, :issue, name, redmine_issue, options
    end

    def link_to_kuba_account(name, account_number, options = {})
      link_to_application :kuba, :account, name, account_number, options
    end

    def link_to_wiki_customer(name, login_name, options = {})
      link_to_application :wiki, :customer, name, login_name, options
    end

    def link_to_migration_item(name, item_id, options = {})
      link_to_application :yimin, :item, name, item_id, options
    end

    private

    def link_to_application(app, entity, name, reference, options = {})
      options.reverse_merge! target: '_blank'
      link_to name, nine_app_url(app, entity, reference), options
    end

    def strategy_from_app_name(app_name, entity)
      raise(InvalidAppError, "Invalid app #{app_name}") unless valid_app?(app_name)
      Bonanza::LinkStrategies.const_get(app_name.capitalize).new(entity)
    end

    def valid_app?(app_name)
      return false if app_name.nil?
      Bonanza::LinkStrategies.constants.include?(app_name.capitalize)
    end
  end
end
