# frozen_string_literal: true

require 'active_support/core_ext'
require 'action_view/helpers'

require 'bonanza/link_helpers'

module Bonanza
  module UrnLinkHelpers
    include Bonanza::LinkHelpers

    URN_APPS = {
      server: :stats,
      vserver: :stats,
      virtdomain: :ninevirt,
      rack: :stats,
      port: :stats,
      :'ip:v4:subnet'           => :stats,
      :'ip:v6:subnet'           => :stats,
      :'aws:account'            => :mojito,
      :'openshift:project:es34' => :openshift
    }.freeze

    def link_to_urn(name, urn, options = {})
      urn  = ::InfrastructureClient::Urn.new(urn) unless urn.is_a?(::InfrastructureClient::Urn)
      name = urn.short if name.blank?

      url = urn.valid? ? url_for_urn(urn) : ''

      options.reverse_merge! target: '_blank'
      link_to name, url, options
    end

    def url_for_urn(urn)
      nine_app_url URN_APPS[urn.type.to_sym], urn.type.to_sym, urn.id
    rescue Bonanza::LinkHelpers::InvalidAppError
      ''
    end
  end
end
