require 'active_model'
require 'simpleidn'

module Bonanza
  module Validators
    class DomainNameValidator < ActiveModel::EachValidator
      DOMAIN_REGEX = /^((?=[a-z0-9-]{1,63}\.)(xn--(--)?)?[a-z0-9]+(-[a-z0-9]+)*\.)+((xn--)?[a-z0-9]{2,63})$/

      def validate_each(record, attribute, value)
        punycode = SimpleIDN.to_ascii(value.to_s)
        return true if punycode =~ DOMAIN_REGEX
        record.errors.add attribute, :not_a_valid_domain_name
      end
    end
  end
end
